<template>
  <div>
    <iframe
      ref="windowiframe"
      :src="src"
      frameborder="0"
      class="iframe-wapper"
      scrolling="auto"
      align="absmiddle"
    />
  </div>
</template>
<script>
import Iframe from '@/views/Manage/components/Iframe.vue'
import {floorHeightMap, areaFloorMap} from './areaFloorMap.js'
export default {
  name: 'IRSMap',
  props: {
    // 弹窗DomElementId
    infoElementId: {
      type: String,
      default: ''
    }
  },
  components: {
    Iframe
  },
  mounted() {
    window.addEventListener('message', event => {
      // 初始化viewer  Cesium infoDialog 如果有弹窗监听页面移动
      if (event.data === 'viewerloaded') {
        this.$nextTick(() => {
          this.viewer =
            (this.$refs.windowiframe &&
              this.$refs.windowiframe.contentWindow &&
              this.$refs.windowiframe.contentWindow.viewer) ||
            null

          this.Cesium =
            (this.$refs.windowiframe &&
              this.$refs.windowiframe.contentWindow &&
              this.$refs.windowiframe.contentWindow.Cesium) ||
            null

          if (this.Cesium && this.infoElementId) {
            this.infoDialog = new this.Cesium.InfoBox(
              document.getElementById(this.infoElementId)
            )
          }

          if (this.viewer && this.infoElementId) {
            this.viewer.scene.postRender.addEventListener(this.updateInfoBox)
            this.viewClickHandler()
          }
        })
      }
    })
  },
  data() {
    return {
      //   src: '/cesiumMap/IRSMap/#/mainView', // 本地
      src: window.context.IRSMap + '/IRSMap/#/mainView', // 部署
      viewer: null,
      Cesium: null,
      infoDialog: null,
      entityPosition: null,
      clickPosition: null,
      currentPointsEntity: [], // 点位Entity集合
      defaultIcon: 'position.png',
      currentRegion: [] // 当前区域
    }
  },
  beforeDestroy() {
    this.viewer.scene.postRender.removeEventListener(this.updateInfoBox)
  },
  methods: {
    // 移除点位
    removePoints() {
      if (this.viewer && this.viewer.entities) {
        this.viewer.entities.removeAll()
        this.currentPointsEntity = []
      }
    },
    // 更新弹窗屏幕位置
    updateInfoBox() {
      try {
        if (this.entityPosition !== null) {
          this.modalSetting(this.entityPosition)
        }
      } catch (error) {
        console.log(error)
      }
    },
    // 添加点位
    addPoints(data) {
      // 添加点位
      data.map(pointItem => {
        const Entity = this.viewer.entities.add({
          id: pointItem.id,
          properties: {...pointItem},
          position: this.Cesium.Cartesian3.fromDegrees(
            pointItem.lgtd,
            pointItem.lttd,
            this.getPointHeight(pointItem)
          ),
          billboard: {
            image: require('@/assets/img/' +
              (pointItem.icon || this.defaultIcon)),
            scale: 0.16,
            horizontalOrigin: this.Cesium.HorizontalOrigin.CENTER,
            verticalOrigin: this.Cesium.VerticalOrigin.BOTTOM
          },
          label: {
            text: pointItem.label,
            font: '16px Helvetica',
            fillColor: this.Cesium.Color.WHITE,
            showBackground: true,
            backgroundColor: this.Cesium.Color(0, 0, 0, 0.8),
            horizontalOrigin: this.Cesium.HorizontalOrigin.CENTER,
            verticalOrigin: this.Cesium.VerticalOrigin.BOTTOM,
            pixelOffset: new this.Cesium.Cartesian2(0, -34)
          }
        })
        this.currentPointsEntity.push(Entity)
      })
    },
    viewClickHandler() {
      // 为点位添加点击事件，监听viewer的pick事件
      this.viewer.screenSpaceEventHandler.setInputAction(movement => {
        // 获取点击位置
        const pickedObject = this.viewer.scene.pick(movement.position)

        // 获取点击位置的经纬度坐标
        this.clickPosition = this.transformPointsToJWH(pickedObject)

        this.$emit('click-screen', {
          position: this.clickPosition
        })

        let findEntity = false
        if (this.currentPointsEntity && this.currentPointsEntity.length) {
          for (let entity of this.currentPointsEntity) {
            if (pickedObject.id === entity) {
              // 点击的点位
              this.entityPosition = entity.position.getValue(
                this.viewer.clock.currentTime
              )

              // 获取点击位置的屏幕坐标
              let screenPosition = this.transformPointsToWindow(
                this.entityPosition
              )

              this.$emit('click-point', {
                id: entity.id,
                data: entity.properties,
                position: screenPosition
              })
              findEntity = true
              break
            }
          }
        }
        findEntity && this.infoElementId ? null : this.$emit('hiddenPop')
      }, this.Cesium.ScreenSpaceEventType.LEFT_CLICK)
    },
    // 根据区域获取高度
    getPointHeight(point) {
      let height = 256.13
      if (point && point.height) {
        height = point.height
      } else if (point && point.areaCode) {
        const floor = areaFloorMap[point.areaCode]
        height = (floor && floorHeightMap[floor]) || 256.13
      }
      return height
    },
    // 弹窗位置移动设置
    modalSetting(entityPosition) {
      let screenPosition = this.Cesium.SceneTransforms.wgs84ToWindowCoordinates(
        this.viewer.scene,
        entityPosition
      )
      if (screenPosition) {
        let screenPosition = this.transformPointsToWindow(this.entityPosition)
        this.$emit('move-point', {
          position: screenPosition
        })
      }
    },
    // 转换为屏幕坐标
    transformPointsToWindow(_position) {
      let position = {
        x: '',
        y: ''
      }

      let screenPosition =
        _position &&
        this.Cesium.SceneTransforms.wgs84ToWindowCoordinates(
          this.viewer.scene,
          _position
        )

      if (screenPosition) {
        position.x = parseInt(screenPosition.x) || ''
        position.y = parseInt(screenPosition.y) || ''
      }
      return position
    },
    // 转换为经纬度、高度坐标 拾取位置
    transformPointsToJWH(pickedObject) {
      let position = {
        longitude: '',
        latitude: '',
        height: ''
      }
      let pickedPoint =
        pickedObject && this.Cesium.Cartographic.fromCartesian(pickedObject)
      if (pickedPoint) {
        position.longitude =
          this.Cesium.Math.toDegrees(pickedPoint.longitude).toFixed(6) || ''
        position.latitude = this.Cesium.Math.toDegrees(
          pickedPoint.latitude || ''
        ).toFixed(6)
        position.height = pickedPoint.height.toFixed(6) || ''
      }

      return position
    },
    // 绘制区域时根据区域编码映射找文件
    findJsonFile(regionCode) {
      let fileName = ''
      if (regionCode) {
        fileName = areaFloorMap[regionCode]
      }
      return fileName
    },
    // 图层显示切换
    updateFloorVisibility(layerId, modelName) {
      const layers = this.viewer.scene.primitives._primitives
      for (let layer of layers) {
        if (
          layer.resource &&
          layer.resource._ionEndpoint &&
          layer.resource._ionEndpoint.assetNumId == layerId
        ) {
          let expression = {conditions: []}
          modelName.forEach(name => {
            expression.conditions.push(["${name} === '" + name + "'", true])
          })
          expression.conditions.push(['true', 'false'])
          let style = new this.Cesium.Cesium3DTileStyle()
          style.show = expression
          layer.style = style
          break
        }
      }
    },
    // 视角跳转到区域对应图层
    flyToFloor(floor) {
      switch (floor) {
        case 'NorthB1':
          this.updateFloorVisibility('38402555338565', ['BZ_B1F_01_LV4'])
          break
        case 'NorthB2':
          this.updateFloorVisibility('38402555338565', ['BZ_B2F_01_LV3'])
          break
        case 'NorthB3':
          this.updateFloorVisibility('38402555338565', [
            'BZ_B3F_01_LV2',
            'HCLZ_B3F_01_LV2',
            'HXZCQB_B3F_01_LV2',
            'N_B3F_01_LV2',
            'ZJBZ_B3F_01_LV2'
          ])
          break
        case 'SouthB1':
          this.updateFloorVisibility('38402555338565', [
            'BZ_B3F_01_LV2',
            'HCLZ_B3F_01_LV2',
            'HXZCQB_B3F_01_LV2',
            'N_B3F_01_LV2',
            'ZJBZ_B3F_01_LV2'
          ])
          break
        case 'SouthB2':
          this.updateFloorVisibility('38402555338565', ['HCLZ_B3F_01_LV2'])
          break
        default:
          this.updateFloorVisibility('38402555338565', [
            'BZ_TOPF_001_LV5',
            'BZ_TOPF_002_LV5',
            'BZ_TOPF_003_LV5',
            'BZ_TOPF_004_LV5',
            'BZ_TOPF_005_LV5',
            'BZ_TOPF_006_LV5',
            'BZ_TOPF_007_LV5',
            'BZ_TOPF_008_LV5'
          ])
          break
      }
    },
    // 显示区域
    showRegion(regionCode) {
      let fileName = this.findJsonFile(regionCode)
      if (!fileName) {
        this.$message.error('该区域暂无数据')
      } else {
        // jsonData一定要json文件
        fileName.indexOf('TOP') > 1 ? (fileName = 'SquareTop') : ''

        const floor = areaFloorMap[regionCode]
        const height = (floor && floorHeightMap[floor]) || 256.13

        const geoJSON = this.Cesium.GeoJsonDataSource.load(
          require(`@/assets/IRSMapRegion/${fileName}.json`),
          {
            stroke: this.Cesium.Color.fromCssColorString('#1e6fdc'),
            fill: this.Cesium.Color.fromCssColorString('#1e6fdc').withAlpha(
              0.6
            ),
            strokeWidth: 3,
            markerSymbol: '?'
          }
        )
        geoJSON.then(dataSource => {
          const region = this.viewer.dataSources.add(dataSource)
          var entities = dataSource.entities.values

          console.log(entities.length, dataSource, 'dataSourcedataSource')

          for (var i = 0; i < entities.length; i++) {
            const entity = entities[i]
            if (entity.properties.BM._value === regionCode) {
              entity.polygon.height = height + 5 // 设置底部高度
              entity.polygon.outline = true
              entity.polygon.outlineWidth = 3 // 设置线宽
              entity.show = true
            } else {
              entity.show = false // 隐藏不匹配的要素
            }
          }

          this.currentRegion.push(region)
        })

        this.viewer.flyTo(geoJSON)
        this.flyToFloor(floor)
      }
    },
    // 移除区域
    removeRegion() {
      if (this.currentRegion.length) {
        this.viewer.dataSources.removeAll()
      }
    }
  }
}
</script>
<style lang="scss" scoped>
.iframe-wapper {
  width: 100%;
  height: 100%;
  text-align: center;
  overflow: hidden;
  position: relative;
}
</style>
